-----------------------------------------------------------------------------
-- |
-- Module      :  Main.hs
-- Copyright   :  (c) Thomas Ilgner 2012
-- Stability   :  provisional
-- Portability :  portable
--
-----------------------------------------------------------------------------
module Main where

import Dpfs.DavisPutnamFiniteSets
import Tests.Tests (generate_queens,generate_nqueens_cyl,generate_superQueens,showQueens,showSudoku,sudokuXxX,sudokuHyper,sudokuNonomino,sudokuHyper1,sudokuX,samuraiSudoku,sudokuXxX16)
import qualified Data.IntSet as Set
import qualified Data.IntMap.Strict as IntMap
import Data.List
import Data.Maybe

import System.Environment
import System.Exit
import Control.Exception.Base

f_print n = do putStrLn("one possible model for the " ++ show n ++ "-QueensProblem: ")               
               putStrLn (show (dpfsSat 0 n (drop n (generate_queens n n))))
               
f_printP n = do putStrLn("one possible model for the " ++ show n ++ "-QueensProblem: ")
                putStrLn (show $ showQueens ((dpfsSat 0 n) (generate_queens n n)))
               
f_print1 n = do putStrLn("one possible model for the " ++ show n ++ "-SuperQueensProblem: ") 
                putStrLn (show (dpfsSat 0 n (generate_superQueens n n)))
                
f_printPS n = do putStrLn("one possible model for the " ++ show n ++ "-SuperQueensProblem: ")
                 putStrLn (show $ showQueens (dpfsSat 0 n (generate_superQueens n n)))                
                
f_print2 n = do putStrLn("one possible model for the " ++ show n ++ "-QueensProblemTorus: ") 
                putStrLn (show (dpfsSat 0 n (generate_nqueens_cyl n n)))
                
f_print4 n = do putStrLn("one possible model for the " ++ show n ++ "-QueensProblem: ")
		putStrLn ("Total Number of Solutions: " ++ show (length $ dpfsSatAllSolutions 0 n (generate_queens n n)))  

f_print5 n = do putStrLn("one possible model for the " ++ show n ++ "-SuperQueensProblem: ")
		putStrLn ("Total Number of Solutions: " ++ show (length $ dpfsSatAllSolutions 0 n (generate_superQueens n n)))
    
main = do    
    args <- getArgs
    parse args
    putStr "\n"
                 
parse ["-h"] = usage   >> exit
parse ["-v"] = version >> exit

parse ("-pq":fs) = f_printP (read (head fs))
parse ("-q":fs) = f_print (read (head fs))
parse ("-qA":fs) = f_print4 (read (head fs))
parse ("-sq":fs) = f_print1 (read (head fs))
parse ("-sqA":fs) = f_print5 (read (head fs))
parse ("-psq":fs) = f_printPS (read (head fs))
parse ("-tq":fs) = f_print2 (read (head fs))

parse ("-l":fs)  = print $ showSudoku 9 $ dpfsSat 1 9 $ sudokuHyper1 9 9
parse ("-l1":fs)  = print $ showSudoku 9 $ dpfsSat 1 9 $ sudokuHyper 9 9
parse ("-l2":fs)  = print $ showSudoku 9 $ dpfsSat 1 9 (sudokuNonomino 9 9)
parse ("-l3":fs)  = print $ showSudoku 9 $ dpfsSat 1 9 (sudokuX 9 9)
parse ("-l4":fs)  = print $ dpfsSat 1 9 (samuraiSudoku 9 9)
parse ("-l5":fs)  = print $ dpfsSat 1 9 (sudokuXxX16 9 9)
parse _     = usage   >> exit


usage   = do putStrLn "Usage: dpfs [-vh] [-fs size of finite Set] [proportional logic formula ..]"
             putStrLn "Parameters:"
             putStrLn "-h Help"
             putStrLn "-v Version"
             putStrLn "-p Print Queens nxn table"
             putStrLn "Examples:"
             putStrLn "N-QueensProblem -> dpfs -q [number for nxn]"
             putStrLn "N-SuperQueensProblem -> dpfs -sq [number for nxn]"
             putStrLn "N-QueensProblemTorus -> dpfs -tq [number for nxn]"
             
             putStrLn "Hypersudoku (teilweise gefüllt) -> dpfs -l"
             putStrLn "Hypersudoku -> dpfs -l1"
             putStrLn "Nonominosudoku -> dpfs -l2"
             putStrLn "X-Sudoku -> dpfs -l3"
             putStrLn "Samurai-Sudoku -> dpfs -l4"
version = putStrLn "Haskell Davis Putnam with finite sets - 0.1"
exit    = exitWith ExitSuccess
die     = exitWith (ExitFailure 1)