{-# LANGUAGE BangPatterns, CPP #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Tests.Test.hs
-- Copyright   :  (c) Thomas Ilgner 2012
-- Stability   :  provisional
-- Portability :  portable
--
-- Test Modul
-----------------------------------------------------------------------------
module Tests.Tests ( 
	    
	    -- * Generatoren
	     generate_queens
	    ,generate_superQueens
	    ,generate_nqueens_cyl
	    ,sudokuXxX
	    ,sudokuHyper
	    ,sudokuHyper1
	    ,sudokuNonomino
	    ,sudokuX
	    ,samuraiSudoku
	    ,sudokuXxX16
	    
	    -- * Utilities
	    ,showQueens
	    ,showSudoku
	    
	    ) where

import Dpfs.DavisPutnamFiniteSets
import Dpfs.Parser
import Dpfs.Simp
import Data.List
import qualified Data.IntSet as Set
import qualified Data.IntMap.Strict as IntMap
import Data.StrictPair
import Data.Maybe
import Control.Monad
---------------------------------------------------------------------------------------------------
-- generates clause set for n-queens-problem
generate_queens :: Int -> Int -> [[Atom]]
generate_queens !n !m = [[Xin x (Set.fromList [1..m])] | x <- [1..n]] ++ 
		        [[Xin k (tS t n) ,Xin z (tS2 k t z n) ] | k<-[1..n], t<-[1..n], z<-[1..n], k /=z ]
  where
    tS :: Int -> Int -> Set.IntSet
    tS x y = let !x1 = x `seq` (Set.singleton x)
		 !x2 = y `seq` (Set.fromList [1..y])
	     in 
	       Set.difference x2 x1
    tS2 :: Int -> Int -> Int -> Int -> Set.IntSet
    tS2 k t z y = let !y'   = k `seq` t `seq` z `seq` y ` seq` Set.foldl' (\l x -> if ( (k == z) || ( t == x) || (k-z == t-x) || ((k-z) == (x-t)) ) then x : l else l ) [] (Set.fromList [1..y])
		      !y''  = Set.fromList y'
		      !y''' = Set.fromList [1..y]
		     in
		      Set.difference (Set.fromList [1..y]) y''  
---------------------------------------------------------------------------------------------------
-- generates clause set for n-SuperQueens-Problem
generate_superQueens !n !m =
        [[Xin x (Set.fromList [1..m])] | x <- [1..n]] ++
        [ [Xin k (tS t n) ,Xin z (tS1 k t z n) ] | k<-[1..n], t<-[1..n], z<-[1..n], k /=z ]
  where 
    tS x y = let !x1 = x `seq` (Set.singleton x)
		 !x2 = y `seq` (Set.fromList [1..y])
	     in (Set.difference x2 x1)
    tS1 k t z y = let new = k `seq` t `seq` z `seq` y ` seq` foldl' (\l x -> if ( (k == z || t == x  || k-z == t-x || ((k-z) == (x-t)) || ((k-z == 2 || z-k == 2) && (t-x == 1 || x-t == 1) ) || ((k-z == 1 || z-k == 1) && (t-x == 2 || x-t == 2) )  ) ) then x : l else l ) [] ([1..y])
		      new1 = Set.fromList [1..y]
		      new2 = Set.fromList new
		  in
		    new `seq` new1 `seq` new2 `seq` (Set.difference new1 new2)
---------------------------------------------------------------------------------------------------
-- generates clause set for n-Queens-Problem (Torus)
generate_nqueens_cyl !n !m
  | ((n `mod` 6) == 1) = [[Xin x (Set.fromList [1..m])] | x <- [1..n]] ++ [ [Xin k (tS t n) ,Xin z (tS1 k t z n) ] | k<-[1..n], t<-[1..n], z<-[1..n], k /=z ]
  | otherwise = error "keine Lösung"
  where 
    tS x y = let !x1 = x `seq` (Set.singleton x)
		 !x2 = y `seq` (Set.fromList [1..y])
	     in (Set.difference x2 x1)
    tS1 k t z y = let new = k `seq` t `seq` z `seq` y ` seq` foldl' (\l x -> if ( ((k-z) `mod` n  == (t-x) `mod` n || k == z || t == x  || (k-z) `mod` n == (x-t) `mod` n) ) then x : l else l ) [] ([1..y])
		      new1 = Set.fromList [1..y]
		      new2 = Set.fromList new
		  in
		    new `seq` new1 `seq` new2 `seq` (Set.difference new1 new2)
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------

set = concat [ [x*10+1..x*10+4] | x<- [1..4]]

countElems3 = foldl' (\map k -> IntMap.insertWith (+) k 1 map) IntMap.empty

data Sudoku = Sudokustring String
instance Show Sudoku
   where show (Sudokustring x) = x

showSudoku n model = Sudokustring (showSudoku' n 1 model)
showSudoku' n z model = if z > n then ""
				 else showSudoku'' n z model ++ ('\n': showSudoku' n (z+1) model)
showSudoku'' n z model = showSudoku''' n z 1 model    
showSudoku''' n z s model = if s > n then "" else (show (head $ Set.toList k)) ++ " " ++ showSudoku''' n z (s+1) model
  where k = fromJust $ IntMap.lookup ((n*(9-z))+s) model

data Queens = Queensstring String
instance Show Queens
   where show (Queensstring x) = x

showQueens :: IntMap.IntMap Set.IntSet -> Queens  
showQueens model = Queensstring (showQueensr (IntMap.size model) 1 model)
showQueensr n z model = if z > n then "" 
      else showQueensra n z model ++ ('\n': showQueensr n (z+1) model)
showQueensra n z model = showQueensrar n z 1 model    
showQueensrar n z s model = if s > n then "" else
    if set == s then "D " ++ showQueensrar n z (s+1) model
    else "- " ++ showQueensrar n z (s+1) model
    where set = setToInt z model     
          
printlistn i  [Queensstring x] = (show i) ++ (". \n" ++ x)
printlistn i  ((Queensstring  x):xs) =  (show i) ++ ((". \n") ++ (x ++  (printlistn (i+1) xs)))


-- Tests CNF Umwandlung --

testCNF1 = numPropSubs 4 $ parseProp "(x1 ELEM {1,2,3,4}) /\\ (x2 ELEM {1,2,3,4}) /\\ (x3 ELEM {1,2,3,4}) /\\ (x4 ELEM {1,2,3,4})"
testCNF2 = let (subs,expr) =  numPropSubs 4 $ parseProp queens_prop 
           in 
           dpcnf 4 $ cnf 4 expr
           
queens_prop = ("(x1 ELEM {1,2,3,4}) /\\ (x2 ELEM {1,2,3,4}) /\\ (x3 ELEM {1,2,3,4}) /\\ (x4 ELEM {1,2,3,4}) /\\ " ++
               "((x2 ELEM {1}) => (x1 ELEM {2,3,4})) /\\ ((x2 ELEM {1}) => (x3 ELEM {2,3,4})) /\\ ((x2 ELEM {1}) => (x4 ELEM {2,3,4})) /\\ " ++ -- Zeile,Spalte x2
               "((x2 ELEM {2}) => (x1 ELEM {1,3,4})) /\\ ((x2 ELEM {2}) => (x3 ELEM {1,3,4})) /\\ ((x2 ELEM {2}) => (x4 ELEM {1,3,4})) /\\ " ++
               "((x2 ELEM {3}) => (x1 ELEM {1,2,4})) /\\ ((x2 ELEM {3}) => (x3 ELEM {1,2,4})) /\\ ((x2 ELEM {3}) => (x4 ELEM {1,2,4})) /\\ " ++
               "((x2 ELEM {4}) => (x1 ELEM {1,2,3})) /\\ ((x2 ELEM {4}) => (x3 ELEM {1,2,3})) /\\ ((x2 ELEM {4}) => (x4 ELEM {1,2,3})) /\\ " ++
               
               "((x2 ELEM {1}) => (x1 ELEM {1,3,4})) /\\ ((x2 ELEM {1}) => (x3 ELEM {1,3,4})) /\\ ((x2 ELEM {1}) => (x4 ELEM {1,2,4})) /\\ " ++ -- Diagonale
               "((x2 ELEM {2}) => (x1 ELEM {1,2,4})) /\\ ((x2 ELEM {2}) => (x3 ELEM {2,4}  )) /\\ ((x2 ELEM {2}) => (x4 ELEM {1,2,3})) /\\ " ++
               "((x2 ELEM {3}) => (x1 ELEM {1,3}  )) /\\ ((x2 ELEM {3}) => (x3 ELEM {1,3}  )) /\\ ((x2 ELEM {3}) => (x4 ELEM {2,3,4})) /\\ " ++
               "((x2 ELEM {4}) => (x1 ELEM {1,2,4})) /\\ ((x2 ELEM {4}) => (x3 ELEM {1,2,4})) /\\ ((x2 ELEM {4}) => (x4 ELEM {1,3,4})) /\\ " ++
               
               "((x1 ELEM {1}) => (x2 ELEM {2,3,4})) /\\ ((x1 ELEM {1}) => (x3 ELEM {2,3,4})) /\\ ((x1 ELEM {1}) => (x4 ELEM {2,3,4})) /\\ " ++ -- Zeile,Spalte x1 
               "((x1 ELEM {2}) => (x2 ELEM {1,3,4})) /\\ ((x1 ELEM {2}) => (x3 ELEM {1,3,4})) /\\ ((x1 ELEM {2}) => (x4 ELEM {1,3,4})) /\\ " ++
               "((x1 ELEM {3}) => (x2 ELEM {1,2,4})) /\\ ((x1 ELEM {3}) => (x3 ELEM {1,2,4})) /\\ ((x1 ELEM {3}) => (x4 ELEM {1,2,4})) /\\ " ++
               "((x1 ELEM {4}) => (x2 ELEM {1,2,3})) /\\ ((x1 ELEM {4}) => (x3 ELEM {1,2,3})) /\\ ((x1 ELEM {4}) => (x4 ELEM {1,2,3})) /\\ " ++
               
               "((x1 ELEM {1}) => (x2 ELEM {1,3,4})) /\\ ((x1 ELEM {1}) => (x3 ELEM {1,2,4})) /\\ ((x1 ELEM {1}) => (x4 ELEM {1,2,3}  )) /\\ " ++ -- Diagonale
               "((x1 ELEM {2}) => (x2 ELEM {2,4}  )) /\\ ((x1 ELEM {2}) => (x3 ELEM {1,2,3})) /\\ ((x1 ELEM {2}) => (x4 ELEM {1,2,3,4})) /\\ " ++
               "((x1 ELEM {3}) => (x2 ELEM {1,3}  )) /\\ ((x1 ELEM {3}) => (x3 ELEM {2,3,4})) /\\ ((x1 ELEM {3}) => (x4 ELEM {1,2,3,4})) /\\ " ++
               "((x1 ELEM {4}) => (x2 ELEM {1,2,4})) /\\ ((x1 ELEM {4}) => (x3 ELEM {1,3,4})) /\\ ((x1 ELEM {4}) => (x4 ELEM {2,3,4}  )) /\\ " ++
               
               "((x3 ELEM {1}) => (x1 ELEM {2,3,4})) /\\ ((x3 ELEM {1}) => (x2 ELEM {2,3,4})) /\\ ((x3 ELEM {1}) => (x4 ELEM {2,3,4})) /\\ " ++ -- Zeile,Spalte x3
               "((x3 ELEM {2}) => (x1 ELEM {1,3,4})) /\\ ((x3 ELEM {2}) => (x2 ELEM {1,3,4})) /\\ ((x3 ELEM {2}) => (x4 ELEM {1,3,4})) /\\ " ++
               "((x3 ELEM {3}) => (x1 ELEM {1,2,4})) /\\ ((x3 ELEM {3}) => (x2 ELEM {1,2,4})) /\\ ((x3 ELEM {3}) => (x4 ELEM {1,2,4})) /\\ " ++
               "((x3 ELEM {4}) => (x1 ELEM {1,2,3})) /\\ ((x3 ELEM {4}) => (x2 ELEM {1,2,3})) /\\ ((x3 ELEM {4}) => (x4 ELEM {1,2,3})) /\\ " ++
               
               "((x3 ELEM {1}) => (x1 ELEM {1,2,4})) /\\ ((x3 ELEM {1}) => (x2 ELEM {1,3,4})) /\\ ((x3 ELEM {1}) => (x4 ELEM {1,3,4})) /\\ " ++ -- Diagonale
               "((x3 ELEM {2}) => (x1 ELEM {1,2,3})) /\\ ((x3 ELEM {2}) => (x2 ELEM {2,3,4})) /\\ ((x3 ELEM {2}) => (x4 ELEM {2,4}  )) /\\ " ++
               "((x3 ELEM {3}) => (x1 ELEM {2,3,4})) /\\ ((x3 ELEM {3}) => (x2 ELEM {1,3}  )) /\\ ((x3 ELEM {3}) => (x4 ELEM {1,3}  )) /\\ " ++
               "((x3 ELEM {4}) => (x1 ELEM {1,3,4})) /\\ ((x3 ELEM {4}) => (x2 ELEM {1,2,4})) /\\ ((x3 ELEM {4}) => (x4 ELEM {1,2,4})) /\\ " ++
               
               "((x4 ELEM {1}) => (x1 ELEM {2,3,4})) /\\ ((x4 ELEM {1}) => (x2 ELEM {2,3,4})) /\\ ((x4 ELEM {1}) => (x3 ELEM {2,3,4})) /\\ " ++ -- Zeile,Spalte
               "((x4 ELEM {2}) => (x1 ELEM {1,3,4})) /\\ ((x4 ELEM {2}) => (x2 ELEM {1,3,4})) /\\ ((x4 ELEM {2}) => (x3 ELEM {1,3,4})) /\\ " ++
               "((x4 ELEM {3}) => (x1 ELEM {1,2,4})) /\\ ((x4 ELEM {3}) => (x2 ELEM {1,2,4})) /\\ ((x4 ELEM {3}) => (x3 ELEM {1,2,4})) /\\ " ++
               "((x4 ELEM {4}) => (x1 ELEM {1,2,3})) /\\ ((x4 ELEM {4}) => (x2 ELEM {1,2,3})) /\\ ((x4 ELEM {4}) => (x3 ELEM {1,2,3})) /\\ " ++
               
               "((x4 ELEM {1}) => (x1 ELEM {1,2,3}  )) /\\ ((x4 ELEM {1}) => (x2 ELEM {1,2,4})) /\\ ((x4 ELEM {1}) => (x3 ELEM {1,3,4})) /\\ " ++ -- Diagonale
               "((x4 ELEM {2}) => (x1 ELEM {1,2,3,4})) /\\ ((x4 ELEM {2}) => (x2 ELEM {1,2,3})) /\\ ((x4 ELEM {2}) => (x3 ELEM {2,4}  )) /\\ " ++
               "((x4 ELEM {3}) => (x1 ELEM {1,2,3,4})) /\\ ((x4 ELEM {3}) => (x2 ELEM {2,3,4})) /\\ ((x4 ELEM {3}) => (x3 ELEM {1,3}  )) /\\ " ++
               "((x4 ELEM {4}) => (x1 ELEM {2,3,4}  )) /\\ ((x4 ELEM {4}) => (x2 ELEM {1,3,4})) /\\ ((x4 ELEM {4}) => (x3 ELEM {1,2,4}))")
               
-- --------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Test Kalkül

generateSudokuXin n m = [ [Xin k (tS t m) ,Xin z (tS2 k t z n) ] | k<-[1..n], t<-[1..n], z<-[1..n], k /=z ]
  where 
    tS x y = let x1 = (Set.singleton x)
		 x2 = (Set.fromList [1..y])
	     in Set.difference x2 x1
    tS2 k t z y = let y' = Set.foldl' (\l x -> if ( (k == z) || ( t == x) ) then x : l else l ) [] (Set.fromList [1..y])
		      y'' = Set.fromList y'
		     in
		      Set.difference (Set.fromList [1..y]) y''

generate_Sudoku = [[Calc (Set.fromList [n..(n+3)]) (Set.fromList [1..4])] | n <- [1,5..13]]
		  ++ [[Calc (Set.fromList [n,(n+4)..(n+12)]) (Set.fromList [1..4])] | n <- [1..4]]
		  ++ [[Calc (Set.fromList [1,2,5,6])     (Set.fromList [1..4])]]
		  ++ [[Calc (Set.fromList [3,4,7,8])     (Set.fromList [1..4])]]
		  ++ [[Calc (Set.fromList [9,10,13,14])  (Set.fromList [1..4])]]
		  ++ [[Calc (Set.fromList [11,12,15,16]) (Set.fromList [1..4])]]		  
		  
sudokuXxX n m = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
		++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
		++ [[Calc (Set.fromList [1,2,3,10,11,12,19,20,21])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [4,5,6,13,14,15,22,23,24])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [7,8,9,16,17,18,25,26,27])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [28,29,30,37,38,39,46,47,48])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [31,32,33,40,41,42,49,50,51])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [34,35,36,43,44,45,52,53,54])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [55,56,57,64,65,66,73,74,75])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [58,59,60,67,68,69,76,77,78])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [61,62,63,70,71,72,79,80,81])     (Set.fromList [1..m])]]
		
sudokuXxX16 n m = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
		++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
		
		++ [[Calc (Set.fromList [1,2,3,4,17,18,19,20,33,34,35,36,49,50,51,52])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [5,6,7,8,21,22,23,24,37,38,39,40,53,54,55,56])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [9,10,11,12,25,26,27,28,41,42,43,44,57,58,59,60])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [13,14,15,16,29,30,31,32,45,46,47,48,61,62,63,64])     (Set.fromList [1..m])]]
		
		++ [[Calc (Set.fromList [65,66,67,68,81,82,83,84,97,98,99,100,113,114,115,116])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [69,70,71,72,85,86,87,88,101,102,103,104,117,118,119,120])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [73,74,75,76,89,90,91,92,105,106,107,108,121,122,123,124])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [77,78,79,80,93,94,95,96,109,110,111,112,125,126,127,128])     (Set.fromList [1..m])]]
		
		++ [[Calc (Set.fromList [129,130,131,132,145,146,147,148,161,162,163,164,177,178,179,180])     (Set.fromList [1..m])]]		
		++ [[Calc (Set.fromList [133,134,135,136,149,150,151,152,165,166,167,168,181,182,183,184])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [137,138,139,140,153,154,155,156,169,170,171,172,185,186,187,188])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [141,142,143,144,157,158,159,160,173,174,175,176,189,190,191,192])     (Set.fromList [1..m])]]
		
		++ [[Calc (Set.fromList [193,194,195,196,209,210,211,212,225,226,227,228,241,242,243,244])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [197,198,199,200,213,214,215,216,229,230,231,232,245,246,247,248])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [201,202,203,204,216,217,218,219,233,234,235,236,249,250,251,252])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [205,206,207,208,220,221,222,223,237,238,239,240,253,254,255,256])     (Set.fromList [1..m])]]

-- 9x9 Hypersudoku
sudokuHyper n m = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
		++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
		++ [[Calc (Set.fromList [1,2,3,10,11,12,19,20,21])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [4,5,6,13,14,15,22,23,24])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [7,8,9,16,17,18,25,26,27])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [28,29,30,37,38,39,46,47,48])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [31,32,33,40,41,42,49,50,51])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [34,35,36,43,44,45,52,53,54])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [55,56,57,64,65,66,73,74,75])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [58,59,60,67,68,69,76,77,78])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [61,62,63,70,71,72,79,80,81])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [11,12,13,20,21,22,29,30,31])     (Set.fromList [1..m])]] -- 1
		++ [[Calc (Set.fromList [15,16,17,24,25,26,33,34,35])     (Set.fromList [1..m])]] -- 2
		++ [[Calc (Set.fromList [47,48,49,56,57,58,65,66,67])     (Set.fromList [1..m])]] -- 3
		++ [[Calc (Set.fromList [51,52,53,60,61,62,69,70,71])     (Set.fromList [1..m])]] -- 4

sudokuHyper1 n m = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
		++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
		++ [[Calc (Set.fromList [1,2,3,10,11,12,19,20,21])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [4,5,6,13,14,15,22,23,24])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [7,8,9,16,17,18,25,26,27])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [28,29,30,37,38,39,46,47,48])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [31,32,33,40,41,42,49,50,51])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [34,35,36,43,44,45,52,53,54])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [55,56,57,64,65,66,73,74,75])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [58,59,60,67,68,69,76,77,78])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [61,62,63,70,71,72,79,80,81])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [11,12,13,20,21,22,29,30,31])     (Set.fromList [1..m])]] -- 1
		++ [[Calc (Set.fromList [15,16,17,24,25,26,33,34,35])     (Set.fromList [1..m])]] -- 2
		++ [[Calc (Set.fromList [47,48,49,56,57,58,65,66,67])     (Set.fromList [1..m])]] -- 3
		++ [[Calc (Set.fromList [51,52,53,60,61,62,69,70,71])     (Set.fromList [1..m])]] -- 4
		++ [[Xin 1 (Set.fromList [6])]]
		++ [[Xin 2 (Set.fromList [9])]]
		++ [[Xin 10 (Set.fromList [5])]]
		++ [[Xin 11 (Set.fromList [8])]]
		++ [[Xin 16 (Set.fromList [9])]]
		++ [[Xin 23 (Set.fromList [8])]]
		++ [[Xin 30 (Set.fromList [3])]]
		++ [[Xin 44 (Set.fromList [8])]]
		++ [[Xin 40 (Set.fromList [3])]]
		++ [[Xin 38 (Set.fromList [7])]]
		++ [[Xin 52 (Set.fromList [5])]]
		++ [[Xin 51 (Set.fromList [6])]]
		-- ++ [[Xin 60 (Set.fromList [1])]]
		++ [[Xin 59 (Set.fromList [5])]]
		++ [[Xin 72 (Set.fromList [4])]]
		++ [[Xin 71 (Set.fromList [3])]]
		++ [[Xin 66 (Set.fromList [2])]]
		++ [[Xin 80 (Set.fromList [1])]]
		-- ++ [[Xin 3 (Set.fromList [1..9])]]

test1 = [[Calc (Set.fromList [1..9]) (Set.fromList [1..9])]]
	++ [[Xin 1 (Set.fromList [5])]]
	++ [[Xin 2 (Set.fromList [7])]]
	++ [[Xin 3 (Set.fromList [4])]]
	++ [[Xin 4 (Set.fromList [1])]]
	++ [[Xin 5 (Set.fromList [2])]]
	++ [[Xin 6 (Set.fromList [8])]]
	++ [[Xin 7 (Set.fromList [3])]]
	++ [[Xin 8 (Set.fromList [2])]] 
		
-- 9x9 Nonomino Variante
sudokuNonomino n m = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
		++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
		++ [[Calc (Set.fromList [46,47,55,64,65,66,73,74,75])     (Set.fromList [1..m])]] -- rot
		++ [[Calc (Set.fromList [63,70,71,72,77,78,79,80,81])     (Set.fromList [1..m])]] -- gelb
		++ [[Calc (Set.fromList [53,54,60,61,62,67,68,69,76])     (Set.fromList [1..m])]] -- orange
		++ [[Calc (Set.fromList [37,38,39,40,48,56,57,58,59])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [30,31,32,33,41,49,50,51,52])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [23,24,25,26,34,42,43,44,45])     (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [6,13,14,15,20,21,22,28,29])      (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [1,2,3,4,5,10,11,12,19])     	  (Set.fromList [1..m])]]
		++ [[Calc (Set.fromList [7,8,9,16,17,18,27,35,36])     	  (Set.fromList [1..m])]]
		++ [[Xin 1 (Set.fromList [5])]]
		++ [[Xin 9 (Set.fromList [7])]]
		++ [[Xin 12 (Set.fromList [4])]]
		++ [[Xin 14 (Set.fromList [1])]]
		++ [[Xin 16 (Set.fromList [9])]]
		++ [[Xin 20 (Set.fromList [8])]]
		++ [[Xin 22 (Set.fromList [3])]]
		++ [[Xin 24 (Set.fromList [4])]]
		++ [[Xin 26 (Set.fromList [6])]]
		++ [[Xin 30 (Set.fromList [9])]]
		++ [[Xin 34 (Set.fromList [8])]]
		++ [[Xin 38 (Set.fromList [2])]]
		++ [[Xin 44 (Set.fromList [1])]]
		++ [[Xin 48 (Set.fromList [5])]]
		++ [[Xin 52 (Set.fromList [6])]]
		++ [[Xin 56 (Set.fromList [1])]]
		++ [[Xin 58 (Set.fromList [9])]]
		++ [[Xin 60 (Set.fromList [8])]]
		++ [[Xin 62 (Set.fromList [2])]]
		++ [[Xin 66 (Set.fromList [2])]]
		++ [[Xin 68 (Set.fromList [6])]]
		++ [[Xin 70 (Set.fromList [1])]]
		++ [[Xin 73 (Set.fromList [3])]]
		++ [[Xin 81 (Set.fromList [4])]]

sudokuX m n = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)]] -- Zeilen
	      ++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..n]] -- Spalten
	      ++ [[Calc (Set.fromList [1,2,3,10,11,12,19,20,21])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [4,5,6,13,14,15,22,23,24])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [7,8,9,16,17,18,25,26,27])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [28,29,30,37,38,39,46,47,48])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [31,32,33,40,41,42,49,50,51])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [34,35,36,43,44,45,52,53,54])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [55,56,57,64,65,66,73,74,75])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [58,59,60,67,68,69,76,77,78])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [61,62,63,70,71,72,79,80,81])     (Set.fromList [1..m])]]
	      ++ [[Calc (Set.fromList [1,11,21,31,41,51,61,71,81])     (Set.fromList [1..m])]] -- unten links nach oben rechts
	      ++ [[Calc (Set.fromList [9,17,25,33,41,49,57,65,73])     (Set.fromList [1..m])]] -- unten rechts nach oben links
	      
samuraiSudoku m n = [[Calc (Set.fromList [z..(z+(n-1))]) (Set.fromList [1..m])] | z <- [(n-(n-1)),((n-(n-1))*(n+1))..(n*n)] \\ [1,10,19] ]
		    ++ [[Calc (Set.fromList [1,2,3,4,5,6,379,380,381]) (Set.fromList [1..m])]] -- Zeilen
		    ++ [[Calc (Set.fromList [10,11,12,13,14,15,388,389,390]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [19,20,21,22,23,24,397,398,399]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [z,(z+n)..(z+(n*(n-1)))]) (Set.fromList [1..m])] | z <- [1..6]] -- Spalten
		    ++ [[Calc (Set.fromList [379,388,397,34,43,52,61,70,79]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [380,389,398,35,44,53,62,71,80]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [381,390,399,36,45,54,63,72,81]) (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [1,2,3,10,11,12,19,20,21])     (Set.fromList [1..m])]] -- Regionen
		    ++ [[Calc (Set.fromList [4,5,6,13,14,15,22,23,24])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [28,29,30,37,38,39,46,47,48])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [31,32,33,40,41,42,49,50,51])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [34,35,36,43,44,45,52,53,54])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [55,56,57,64,65,66,73,74,75])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [58,59,60,67,68,69,76,77,78])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [61,62,63,70,71,72,79,80,81])     (Set.fromList [1..m])]]
		    
		    {- 2. Quadrat -}
		    ++ [[Calc (Set.fromList [z..(z+(9-1))]) (Set.fromList [1..9])] | z <- [109,118,127,136,145,154]]		    
		    ++ [[Calc (Set.fromList [385,386,387,85,86,87,88,89,90]) (Set.fromList [1..m])]] -- Zeilen
		    ++ [[Calc (Set.fromList [394,395,396,94,95,96,97,98,99]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [403,404,405,103,104,105,106,107,108]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [z,(z+9)..(z+(9*(9-1)))]) (Set.fromList [1..9])] | z <- [85..90]]
		    ++ [[Calc (Set.fromList [385,394,403,109,118,127,136,145,154]) (Set.fromList [1..m])]] -- Zeilen
		    ++ [[Calc (Set.fromList [386,395,404,110,119,128,137,146,155]) (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [387,396,405,111,120,129,138,147,156]) (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [85,86,87,94,95,96,112,113,114])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [88,89,90,97,98,99,106,107,108])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [109,110,111,118,119,120,127,128,129])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [112,113,114,121,122,123,130,131,132])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [115,116,117,124,125,126,133,134,135])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [136,137,138,145,146,147,154,155,156])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [139,140,141,148,149,150,157,158,159])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [142,143,144,151,152,153,160,161,162])     (Set.fromList [1..m])]]
		    
		    {- 3. Quadrat -}		    
		    ++ [[Calc (Set.fromList [z..(z+(9-1))]) (Set.fromList [1..9])] | z <- [163,172,181,190,199,208]]
		    ++ [[Calc (Set.fromList [217,218,219,220,221,222,325,326,327])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [226,227,228,229,230,231,334,335,336])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [235,236,237,238,239,240,343,344,345])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [z,(z+9)..(z+(9*(9-1)))]) (Set.fromList [1..9])] | z <- [163..168]]
		    ++ [[Calc (Set.fromList [169,178,187,196,205,214,325,334,343])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [170,179,188,197,206,215,326,335,344])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [171,180,189,198,207,216,327,336,345])     (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [163,164,165,172,173,174,181,182,183])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [166,167,168,175,176,177,184,185,186])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [169,170,171,178,179,180,187,188,189])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [190,191,192,199,200,201,208,209,210])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [193,194,195,202,203,204,211,212,213])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [196,197,198,205,206,207,214,215,216])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [217,218,219,226,227,228,235,236,237])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [220,221,222,229,230,231,238,239,240])     (Set.fromList [1..m])]]
		    
		    {- 4. Quadrat -}
		    ++ [[Calc (Set.fromList [z..(z+(9-1))]) (Set.fromList [1..9])] | z <- [244,253,262,271,280,289]]
		    ++ [[Calc (Set.fromList [331,332,333,301,302,303,304,305,306])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [340,341,342,310,311,312,313,314,315])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [349,350,351,319,320,321,322,323,324])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [z,(z+9)..(z+(9*(9-1)))]) (Set.fromList [1..9])] | z <- [247..252]]
		    ++ [[Calc (Set.fromList [244,253,262,271,280,289,331,340,349])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [245,254,263,272,281,290,332,341,350])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [246,255,264,273,282,291,333,342,351])     (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [244,245,246,253,254,255,262,263,264])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [247,248,249,256,257,258,265,266,267])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [250,251,252,259,260,261,268,269,270])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [271,272,273,280,281,282,289,290,291])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [274,275,276,283,284,285,292,293,294])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [277,278,279,286,287,288,295,296,297])     (Set.fromList [1..m])]]		    
		    ++ [[Calc (Set.fromList [301,302,303,310,311,312,319,320,321])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [304,305,306,313,314,315,322,323,324])     (Set.fromList [1..m])]]
		    
		    {- 5. Quadrat -}
		    ++ [[Calc (Set.fromList [z..(z+(9-1))]) (Set.fromList [1..9])] | z <- [325,334..405]]
		    ++ [[Calc (Set.fromList [z,(z+9)..(z+(9*(9-1)))]) (Set.fromList [1..9])] | z <- [325..333]]
		    ++ [[Calc (Set.fromList [325,326,327,334,335,336,343,344,345])     (Set.fromList [1..m])]]		    
		    ++ [[Calc (Set.fromList [328,329,330,337,338,339,346,347,348])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [331,332,333,340,341,342,349,350,351])     (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [352,353,354,361,362,363,370,371,372])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [355,356,357,364,365,366,373,374,375])     (Set.fromList [1..m])]]
		    ++ [[Calc (Set.fromList [358,359,360,367,368,369,376,377,378])     (Set.fromList [1..m])]]
		    
		    ++ [[Calc (Set.fromList [379,380,381,388,389,390,397,398,399])     (Set.fromList [1..m])]] -- 7
		    ++ [[Calc (Set.fromList [382,283,284,292,293,294,400,401,402])     (Set.fromList [1..m])]] -- 8
		    ++ [[Calc (Set.fromList [385,386,387,394,395,396,403,404,405])     (Set.fromList [1..m])]] -- 9
	     
xinSamurai = [[Xin 1 (Set.fromList [1])]] 
	    ++ [[Xin 2 (Set.fromList [3])]]
	    ++ [[Xin 4 (Set.fromList [6])]]
	    ++ [[Xin 5 (Set.fromList [5])]] 
	    ++ [[Xin 381 (Set.fromList [4])]] 
	    ++ [[Xin 11 (Set.fromList [5])]] 
	    ++ [[Xin 12 (Set.fromList [4])]] 
	    ++ [[Xin 13 (Set.fromList [9])]]
	    ++ [[Xin 389 (Set.fromList [2])]] 
	    ++ [[Xin 390 (Set.fromList [6])]]
	    ++ [[Xin 21 (Set.fromList [6])]]
	    ++ [[Xin 397 (Set.fromList [3])]]
	    ++ [[Xin 398 (Set.fromList [5])]]
	    ++ [[Xin 35 (Set.fromList [1])]]
	    ++ [[Xin 36 (Set.fromList [8])]]
	    ++ [[Xin 37 (Set.fromList [2])]]
	    ++ [[Xin 41 (Set.fromList [4])]]
	    ++ [[Xin 45 (Set.fromList [9])]]

	    ++ [[Xin 46 (Set.fromList [6])]]
	    ++ [[Xin 47 (Set.fromList [1])]]

	    ++ [[Xin 56 (Set.fromList [2])]]
	    ++ [[Xin 57 (Set.fromList [9])]]
	    ++ [[Xin 61 (Set.fromList [1])]]

	    ++ [[Xin 64 (Set.fromList [7])]]
	    ++ [[Xin 65 (Set.fromList [6])]]
	    ++ [[Xin 69 (Set.fromList [4])]]
	    ++ [[Xin 70 (Set.fromList [9])]]
	    ++ [[Xin 71 (Set.fromList [8])]]

	    ++ [[Xin 73 (Set.fromList [3])]]
	    ++ [[Xin 77 (Set.fromList [7])]]
	    ++ [[Xin 78 (Set.fromList [9])]]
	    ++ [[Xin 80 (Set.fromList [6])]]
	    ++ [[Xin 81 (Set.fromList [5])]]


	    ++ [[Xin 385 (Set.fromList [5])]]
	    ++ [[Xin 386 (Set.fromList [3])]]
	    ++ [[Xin 85 (Set.fromList [7])]]
	    ++ [[Xin 86 (Set.fromList [4])]]
	    ++ [[Xin 90 (Set.fromList [6])]]

	    ++ [[Xin 395 (Set.fromList [8])]]
	    ++ [[Xin 396 (Set.fromList [9])]]
	    ++ [[Xin 94 (Set.fromList [6])]]
	    ++ [[Xin 98 (Set.fromList [2])]]
	    ++ [[Xin 99 (Set.fromList [5])]]

	    ++ [[Xin 405 (Set.fromList [2])]]
	    ++ [[Xin 106 (Set.fromList [3])]]
	    ++ [[Xin 107 (Set.fromList [1])]]

	    ++ [[Xin 116 (Set.fromList [5])]]
	    ++ [[Xin 117 (Set.fromList [8])]]

	    ++ [[Xin 118 (Set.fromList [2])]]
	    ++ [[Xin 122 (Set.fromList [5])]]
	    ++ [[Xin 126 (Set.fromList [3])]]

	    ++ [[Xin 127 (Set.fromList [9])]]
	    ++ [[Xin 128 (Set.fromList [5])]]

	    ++ [[Xin 137 (Set.fromList [9])]]
	    ++ [[Xin 138 (Set.fromList [6])]]
	    ++ [[Xin 142 (Set.fromList [4])]]

	    ++ [[Xin 145 (Set.fromList [3])]]
	    ++ [[Xin 146 (Set.fromList [7])]]
	    ++ [[Xin 150 (Set.fromList [1])]]
	    ++ [[Xin 151 (Set.fromList [5])]]
	    ++ [[Xin 152 (Set.fromList [6])]]

	    ++ [[Xin 154 (Set.fromList [1])]]
	    ++ [[Xin 158 (Set.fromList [7])]]
	    ++ [[Xin 159 (Set.fromList [6])]]
	    ++ [[Xin 161 (Set.fromList [3])]]
	    ++ [[Xin 162 (Set.fromList [9])]]
	    ------
	    ++ [[Xin 163 (Set.fromList [2])]]
	    ++ [[Xin 164 (Set.fromList [5])]]
	    ++ [[Xin 166 (Set.fromList [9])]]
	    ++ [[Xin 167 (Set.fromList [7])]]
	    ++ [[Xin 171 (Set.fromList [3])]]

	    ++ [[Xin 173 (Set.fromList [1])]]
	    ++ [[Xin 174 (Set.fromList [3])]]
	    ++ [[Xin 175 (Set.fromList [6])]]
	    ++ [[Xin 179 (Set.fromList [5])]]
	    ++ [[Xin 180 (Set.fromList [9])]]

	    ++ [[Xin 183 (Set.fromList [8])]]
	    ++ [[Xin 187 (Set.fromList [2])]]
	    ++ [[Xin 188 (Set.fromList [4])]]

	    ++ [[Xin 197 (Set.fromList [8])]]
	    ++ [[Xin 198 (Set.fromList [1])]]

	    ++ [[Xin 199 (Set.fromList [6])]]
	    ++ [[Xin 203 (Set.fromList [3])]]
	    ++ [[Xin 207 (Set.fromList [4])]]

	    ++ [[Xin 208 (Set.fromList [1])]]
	    ++ [[Xin 209 (Set.fromList [4])]]

	    ++ [[Xin 218 (Set.fromList [2])]]
	    ++ [[Xin 219 (Set.fromList [1])]]
	    ++ [[Xin 325 (Set.fromList [9])]]

	    ++ [[Xin 226 (Set.fromList [4])]]
	    ++ [[Xin 227 (Set.fromList [7])]]
	    ++ [[Xin 231 (Set.fromList [5])]]
	    ++ [[Xin 334 (Set.fromList [6])]]
	    ++ [[Xin 335 (Set.fromList [3])]]

	    ++ [[Xin 235 (Set.fromList [8])]]
	    ++ [[Xin 239 (Set.fromList [2])]]
	    ++ [[Xin 240 (Set.fromList [9])]]
	    ++ [[Xin 344 (Set.fromList [1])]]
	    ++ [[Xin 345 (Set.fromList [5])]]

	    ------
	    ++ [[Xin 244 (Set.fromList [8])]]
	    ++ [[Xin 245 (Set.fromList [6])]]
	    ++ [[Xin 247 (Set.fromList [3])]]
	    ++ [[Xin 248 (Set.fromList [7])]]
	    ++ [[Xin 252 (Set.fromList [1])]]

	    ++ [[Xin 254 (Set.fromList [3])]]
	    ++ [[Xin 255 (Set.fromList [2])]]
	    ++ [[Xin 256 (Set.fromList [1])]]
	    ++ [[Xin 260 (Set.fromList [8])]]
	    ++ [[Xin 261 (Set.fromList [7])]]

	    ++ [[Xin 264 (Set.fromList [1])]]
	    ++ [[Xin 268 (Set.fromList [6])]]
	    ++ [[Xin 269 (Set.fromList [5])]]

	    ++ [[Xin 277 (Set.fromList [4])]]
	    ++ [[Xin 278 (Set.fromList [9])]]

	    ++ [[Xin 280 (Set.fromList [5])]]
	    ++ [[Xin 284 (Set.fromList [3])]]
	    ++ [[Xin 288 (Set.fromList [8])]]

	    ++ [[Xin 289 (Set.fromList [1])]]
	    ++ [[Xin 299 (Set.fromList [8])]]

	    ++ [[Xin 332 (Set.fromList [5])]]
	    ++ [[Xin 333 (Set.fromList [4])]]
	    ++ [[Xin 301 (Set.fromList [1])]]

	    ++ [[Xin 340 (Set.fromList [9])]]
	    ++ [[Xin 341 (Set.fromList [1])]]
	    ++ [[Xin 312 (Set.fromList [7])]]
	    ++ [[Xin 313 (Set.fromList [2])]]
	    ++ [[Xin 314 (Set.fromList [3])]]

	    ++ [[Xin 349 (Set.fromList [6])]]
	    ++ [[Xin 320 (Set.fromList [4])]]
	    ++ [[Xin 321 (Set.fromList [1])]]
	    ++ [[Xin 323 (Set.fromList [9])]]
	    ++ [[Xin 324 (Set.fromList [5])]]

	    ----
	    ++ [[Xin 386 (Set.fromList [8])]]

	    ++ [[Xin 355 (Set.fromList [6])]]
	    ++ [[Xin 357 (Set.fromList [2])]]

	    ++ [[Xin 363 (Set.fromList [7])]]
	    ++ [[Xin 327 (Set.fromList [3])]]

	    ++ [[Xin 373 (Set.fromList [3])]]
	    ++ [[Xin 375 (Set.fromList [4])]]
	    ++ [[Xin 383 (Set.fromList [2])]]